function write_grid( lone,late, lonr,latr, pn,pm, angle,orterr, ...
                           size_x,size_y, cent_lat, NStpr,EWtpr, ...
                             cushn1,xcr1,ycr1, cushn2,xcr2,ycr2, ...
                             cushn3,xcr3,ycr3, cushn4,xcr4,ycr4, ...
                              alpha, tra_lon,tra_lat, flip_xy );
%% Create the grid file

grdname = 'roms_grd.nc';
ncid = netcdf.create(grdname,'64BIT_OFFSET');

%% Define dimensions

[xi_rho,eta_rho]=size(lonr);
xi_r_id  = netcdf.defDim(ncid, 'xi_rho',  xi_rho);
eta_r_id = netcdf.defDim(ncid, 'eta_rho', eta_rho);
r2dgrd = [xi_r_id, eta_r_id];

xi_u=xi_rho-1; eta_v=eta_rho-1;
xi_u_id  = netcdf.defDim(ncid, 'xi_u',  xi_u);
eta_v_id = netcdf.defDim(ncid, 'eta_v', eta_v);
p2dgrd = [xi_u_id, eta_v_id];

%% Create variables and their attributes

sph_vid=netcdf.defVar(ncid, 'spherical', 'NC_CHAR', []);
netcdf.putAtt(ncid, sph_vid, 'long_name', 'Grid type logical switch');
netcdf.putAtt(ncid, sph_vid, 'option_T', 'spherical');

lon_r_vid=netcdf.defVar(ncid, 'lon_rho', 'NC_DOUBLE', r2dgrd);
netcdf.putAtt(ncid, lon_r_vid, 'long_name','longitude of RHO-points');
netcdf.putAtt(ncid, lon_r_vid, 'units', 'degree East');

lat_r_vid=netcdf.defVar(ncid, 'lat_rho', 'NC_DOUBLE', r2dgrd);
netcdf.putAtt(ncid, lat_r_vid, 'long_name','latitude of RHO-points');
netcdf.putAtt(ncid, lat_r_vid, 'units', 'degree North');

lon_p_vid=netcdf.defVar(ncid, 'lon_psi', 'NC_DOUBLE', p2dgrd);
netcdf.putAtt(ncid, lon_p_vid, 'long_name','longitude of PSI-points');
netcdf.putAtt(ncid, lon_p_vid, 'units', 'degree East');

lat_p_vid=netcdf.defVar(ncid, 'lat_psi', 'NC_DOUBLE', p2dgrd);
netcdf.putAtt(ncid, lat_p_vid, 'long_name','latitude of PSI-points');
netcdf.putAtt(ncid, lat_p_vid, 'units', 'degree North');

pm_vid=netcdf.defVar(ncid, 'pm', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, pm_vid, 'long_name', ...
                   'curvilinear coordinate metric in XI-direction');
netcdf.putAtt(ncid, pm_vid, 'units', 'meter-1');

pn_vid=netcdf.defVar(ncid, 'pn', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, pn_vid, 'long_name', ...
                     'curvilinear coordinate metric in ETA-direction');
netcdf.putAtt(ncid, pn_vid, 'units','meter-1');

ang_vid=netcdf.defVar(ncid, 'angle', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, ang_vid, 'long_name', ...
                              'angle between east and XI-directions');
netcdf.putAtt(ncid, ang_vid, 'units', 'degrees');


orterr_vid=netcdf.defVar(ncid, 'orterr', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, orterr_vid, 'long_name', ...
                              'orthogonality error');
netcdf.putAtt(ncid, orterr_vid, 'units', 'degrees');

f_vid=netcdf.defVar(ncid, 'f', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, f_vid, 'long_name','Coriolis parameter at RHO-points');
netcdf.putAtt(ncid, f_vid, 'units', 'second-1');

hraw_vid=netcdf.defVar(ncid, 'hraw', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, hraw_vid, 'long_name','raw bathymetry at RHO-points');
netcdf.putAtt(ncid, hraw_vid, 'units','meter');

h_vid=netcdf.defVar(ncid, 'h', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, h_vid, 'long_name','model bathymetry at RHO-points');
netcdf.putAtt(ncid, h_vid, 'units','meter');

mask_r_vid=netcdf.defVar(ncid, 'mask_rho', 'NC_DOUBLE',  r2dgrd);
netcdf.putAtt(ncid, mask_r_vid, 'long_name', 'land/sea mask at RHO-points');
netcdf.putAtt(ncid, mask_r_vid, 'units','land/water (0/1)');

%% Global attributes

if (flip_xy > 0)
  jj=xi_rho-2; ii=eta_rho-2;   %% <-- reverse order
else
  ii=xi_rho-2; jj=eta_rho-2;
end
settings = ['nx=',num2str(ii),             ' ny=',num2str(jj), ...
       ' size_x=',num2str(size_x/1e3), ' size_y=',num2str(size_y/1e3), ...
                                     ' cent_lat=',num2str(cent_lat), ...
        ' NStpr=',num2str(NStpr),       ' EWtpr=',num2str(EWtpr), ...
       ' cushn1=',num2str(cushn1),       ' xcr1=',num2str(xcr1), ...
                                         ' ycr1=',num2str(ycr1), ...
       ' cushn2=',num2str(cushn2),       ' xcr2=',num2str(xcr2), ...
                                         ' ycr2=',num2str(ycr2), ...
       ' cushn3=',num2str(cushn3),       ' xcr3=',num2str(xcr3), ...
                                         ' ycr3=',num2str(ycr3), ...
       ' cushn4=',num2str(cushn4),       ' xcr4=',num2str(xcr4), ...
                                         ' ycr4=',num2str(ycr4), ...
          ' Lat=',num2str(tra_lat),       ' Lon=',num2str(tra_lon), ...
       ' rotate=',num2str(alpha),     ' flip_xy=',num2str(flip_xy)];

disp([ 'write_grid :: settings ', settings ])

varid=netcdf.getConstant('GLOBAL');
netcdf.putAtt(ncid, varid, 'Title', 'ROMS grid produced by Matlab Grid GUI');
netcdf.putAtt(ncid, varid, 'settings', settings);
netcdf.putAtt(ncid, varid, 'Date', date);

netcdf.endDef(ncid);  %% <-- complete file definition

%% Fill the file with data

disp('writing spherical...')
netcdf.putVar(ncid, sph_vid, 'T');

pi=3.14159265358979323846; rad2deg=180/pi;
disp(['writing lon_r, size = ', num2str(size(latr))])
netcdf.putVar(ncid, lon_r_vid, rad2deg*lonr); %% <-- convert to degrees

disp(['writing lat_r, size = ', num2str(size(latr))])
netcdf.putVar(ncid, lat_r_vid, rad2deg*latr); %% <-- convert to degrees

tmp_p=zeros(xi_u,eta_v);
tmp_p=rad2deg*lone(2:xi_u+1,2:eta_v+1);
disp(['writing lon_p, size of lonp = ',num2str(size(tmp_p))])
netcdf.putVar(ncid, lon_p_vid, tmp_p);

tmp_p=rad2deg*late(2:xi_u+1,2:eta_v+1);
disp(['writing lat_p, size of latp = ',num2str(size(tmp_p))])
netcdf.putVar(ncid, lat_p_vid, tmp_p);
clear tmp_p;

disp(['writing pm, size = ', num2str(size(pm))])
netcdf.putVar(ncid, pm_vid, pm);

disp(['writing pn, size = ', num2str(size(pn))])
netcdf.putVar(ncid, pn_vid, pn);

disp(['writing angle, size = ', num2str(size(angle))])
netcdf.putVar(ncid, ang_vid, rad2deg*angle);

disp(['writing orterr, size = ', num2str(size(orterr))])
netcdf.putVar(ncid, orterr_vid, rad2deg*orterr);

f=4*pi*sin(latr)/(23.9344699*3600); %% <-- 4*pi/T where T is SIDEREAL day
disp(['writing f, size = ', num2str(size(f))])
netcdf.putVar(ncid, f_vid, f);
clear f;

disp('closing file...')
netcdf.close(ncid)
disp('done write_grid')
